@@ -10,7 +10,8 @@
 #' @param ratio.colbar The width ratio of colorbar to the total colorlegend
 #'   (including colorbar, segments and labels).
 #' @param lim.segment Vector (quantile) of length 2, the elements should be in
-#'   [-1,1], giving segments coordinates ranges.
+#'   [0,1], giving segments coordinates ranges. If the value is NULL or "auto",
+#'   then the ranges are derived automatically.
 #' @param align Character, alignment type of labels, \code{"l"} means left,
 #'   \code{"c"} means center and \code{"r"} right.
 #' @param addlabels Logical, whether add text label or not.
@@ -20,19 +21,38 @@
 #' @keywords hplot
 #' @author Taiyun Wei
 #' @export
-colorlegend <- function(colbar, labels, at = NULL,
-    xlim = c(0, 1), ylim = c(0, 1), vertical = TRUE, ratio.colbar = 0.4,
-    lim.segment = NULL, align = c("c", "l", "r"), addlabels = TRUE,
-    ...) {
-
-  if (is.null(at) & addlabels)
+colorlegend <- function(
+  colbar,
+  labels,
+  at = NULL,
+  xlim = c(0, 1),
+  ylim = c(0, 1),
+  vertical = TRUE,
+  ratio.colbar = 0.4,
+  lim.segment = "auto", # NOTE: NULL treated as "auto"
+  align = c("c", "l", "r"),
+  addlabels = TRUE,
+  ...)
+{
+  if (is.null(at) && addlabels) {
     at <- seq(0L, 1L, length = length(labels))
-  if (is.null(lim.segment))
-    lim.segment <- ratio.colbar + c(0, ratio.colbar / 5)
-  if (any(at < 0L) | any(at > 1L))
+  }
+
+  if (any(is.null(lim.segment)) || any(lim.segment == "auto")) {
+    lim.segment <- ratio.colbar + c(0, ratio.colbar * .2)
+  }
+
+  if (any(at < 0L) || any(at > 1L)) {
     stop("at should be between 0 and 1")
-  if (any(lim.segment < 0L) | any(lim.segment > 1L))
+  }
+
+  if (length(lim.segment) != 2) {
+    stop("lim.segment should be a vector of length 2")
+  }
+
+  if (any(lim.segment < 0L) || any(lim.segment > 1L)) {
     stop("lim.segment should be between 0 and 1")
+  }
 
   align <- match.arg(align)
   xgap <- diff(xlim)
@@ -42,42 +62,43 @@ colorlegend <- function(colbar, labels, at = NULL,
   rat2 <- lim.segment
 
   if (vertical) {
+
       at <- at * ygap + ylim[1]
       yyy <- seq(ylim[1], ylim[2], length = len + 1)
-      rect(rep(xlim[1], len), yyy[1:len], rep(xlim[1] +
-          xgap * rat1, len), yyy[-1], col = colbar, border = colbar)
-      rect(xlim[1], ylim[1], xlim[1] + xgap * rat1, ylim[2],
-          border = "black")
-      pos.xlabel <- rep(xlim[1] + xgap * max(rat2, rat1),
-          length(at))
+
+      rect(rep(xlim[1], len), yyy[1:len],
+           rep(xlim[1] + xgap * rat1, len), yyy[-1],
+           col = colbar, border = colbar)
+      rect(xlim[1], ylim[1], xlim[1] + xgap * rat1, ylim[2], border = "black")
       segments(xlim[1] + xgap * rat2[1], at, xlim[1] + xgap * rat2[2], at)
 
       if (addlabels) {
-          if (align == "l")
-              text(x = pos.xlabel, y = at, labels = labels,
-                pos = 4, ...)
-          if (align == "c")
-              text((pos.xlabel + xlim[2]) / 2, y = at, labels = labels, ...)
-          if (align == "r")
-              text(x = xlim[2], y = at, labels = labels,
-                pos = 2, ...)
+        pos.xlabel <- rep(xlim[1] + xgap * max(rat2, rat1), length(at))
+        switch(align,
+          l = text(pos.xlabel, y = at, labels = labels, pos = 4, ...),
+          r = text(xlim[2],    y = at, labels = labels, pos = 2, ...),
+          c = text((pos.xlabel + xlim[2]) / 2, y = at, labels = labels, ...),
+          stop("programming error - should not have reached this line!")
+        )
       }
-  }
+  } else {
 
-  if (!vertical) {
     at <- at * xgap + xlim[1]
     xxx <- seq(xlim[1], xlim[2], length = len + 1)
+
     rect(xxx[1:len], rep(ylim[2] - rat1 * ygap, len),
-         xxx[-1], rep(ylim[2], len), col = colbar, border = colbar)
+         xxx[-1], rep(ylim[2], len),
+         col = colbar, border = colbar)
     rect(xlim[1], ylim[2] - rat1 * ygap, xlim[2], ylim[2], border = "black")
-    pos.ylabel <- rep(ylim[2] - ygap * max(rat2, rat1), length(at))
     segments(at, ylim[2] - ygap * rat2[1], at, ylim[2] - ygap * rat2[2])
 
     if (addlabels) {
+      pos.ylabel <- rep(ylim[2] - ygap * max(rat2, rat1), length(at))
       switch(align,
-       l = text(at, pos.ylabel, labels, pos = 1, ...),
-       c = text(at, (pos.ylabel + ylim[1]) / 2, labels = labels, ...),
-       r = text(at, ylim[1], labels = labels, pos = 2, ...)
+       l = text(x = at, y = pos.ylabel, labels = labels, pos = 1, ...),
+       r = text(x = at, y = ylim[1],    labels = labels, pos = 2, ...),
+       c = text(x = at, y = (pos.ylabel + ylim[1]) / 2, labels = labels, ...),
+       stop("programming error - should not have reached this line!")
       )
     }
   }
